//------------------------------------------------------------------
#property copyright "mladen"
#property link      "www,forex-station.com"
//------------------------------------------------------------------
#property indicator_separate_window
#property indicator_buffers 5
#property indicator_color5  clrSilver
#property indicator_width1  2
#property indicator_width2  1
#property indicator_width3  2
#property indicator_width4  1
#property indicator_width5  3
#property strict

//
//
//
//
//
enum enTimeFrames
{
   tf_cu  = PERIOD_CURRENT, // Current time frame
   tf_m1  = PERIOD_M1,      // 1 minute
   tf_m5  = PERIOD_M5,      // 5 minutes
   tf_m15 = PERIOD_M15,     // 15 minutes
   tf_m30 = PERIOD_M30,     // 30 minutes
   tf_h1  = PERIOD_H1,      // 1 hour
   tf_h4  = PERIOD_H4,      // 4 hours
   tf_d1  = PERIOD_D1,      // Daily
   tf_w1  = PERIOD_W1,      // Weekly
   tf_mn1 = PERIOD_MN1,     // Monthly
   tf_n1  = -1,             // First higher time frame
   tf_n2  = -2,             // Second higher time frame
   tf_n3  = -3              // Third higher time frame
};
//
//

extern enTimeFrames    TimeFrame          = tf_cu;   // Time frame
input int                CCIPeriod       = 50;                 // Cci period
input int                RsxPeriod       = 30;                 // Rsx period
input ENUM_APPLIED_PRICE RsxPrice        = PRICE_CLOSE;        // Rsx price
input int                AtrPeriod       = 30;                 // Atr period
input double             OverSold        = -100;               // Over sold level
input double             OverBought      = 100;                // Over bought level
input color              OverSoldColor   = clrLimeGreen;       // Over sold color
input color              OverBoughtColor = clrOrange;          // Over bought color
input bool               Interpolate     = true;               // Interpolate values on/off?

double cci[],cciUpa[],cciUpb[],cciDna[],cciDnb[],prices[],trend[],slope[],count[];
string indicatorFileName;
#define _mtfCall(_buff,_ind) iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,CCIPeriod,RsxPeriod,RsxPrice,AtrPeriod,OverSold,OverBought,OverSoldColor,OverBoughtColor,_buff,_ind)

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int OnInit()
{
   IndicatorBuffers(9);
      SetIndexBuffer(0,cciUpa,INDICATOR_DATA); SetIndexStyle(0,DRAW_HISTOGRAM,EMPTY,EMPTY,OverSoldColor);
      SetIndexBuffer(1,cciUpb,INDICATOR_DATA); SetIndexStyle(1,DRAW_HISTOGRAM,EMPTY,EMPTY,OverSoldColor);
      SetIndexBuffer(2,cciDna,INDICATOR_DATA); SetIndexStyle(2,DRAW_HISTOGRAM,EMPTY,EMPTY,OverBoughtColor);
      SetIndexBuffer(3,cciDnb,INDICATOR_DATA); SetIndexStyle(3,DRAW_HISTOGRAM,EMPTY,EMPTY,OverBoughtColor);
      SetIndexBuffer(4,cci,   INDICATOR_DATA); SetIndexStyle(4,DRAW_LINE);
      SetIndexBuffer(5,prices);
      SetIndexBuffer(6,trend);
      SetIndexBuffer(7,slope);
      SetIndexBuffer(8,count);
      SetLevelValue(0,OverBought);
      SetLevelValue(1,OverSold);
         
      indicatorFileName = WindowExpertName();
     TimeFrame         = (enTimeFrames)timeFrameValue(TimeFrame);
       
   IndicatorSetString(INDICATOR_SHORTNAME,timeFrameToString(TimeFrame)+" CCI of rsx & atr ("+(string)CCIPeriod+","+(string)RsxPeriod+")");
return(INIT_SUCCEEDED);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//

double work[][13];
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime& time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
{
   int r,i=fmin(rates_total-prev_calculated+1,rates_total-1); count[0]=i;
      if (TimeFrame!=_Period)
      {
         i = (int)fmax(i,fmin(rates_total-1,_mtfCall(8,0)*TimeFrame/_Period));
         for (; i>=0 && !_StopFlag; i--)
         {
             int y = iBarShift(NULL,TimeFrame,time[i]); 
                cciUpa[i] = _mtfCall(0,y);
                cciUpb[i] = _mtfCall(1,y);
                cciDna[i] = _mtfCall(2,y);
                cciDnb[i] = _mtfCall(3,y);
                cci[i]    = _mtfCall(4,y);
                trend[i]  = _mtfCall(6,y);
                slope[i]  = _mtfCall(7,y);
                  
                //
                //
                //
                //
                //
                  
                if (!Interpolate || (i>0 && y==iBarShift(NULL,TimeFrame,time[i-1]))) continue;
                   #define _interpolate(buff) buff[i+k] = buff[i]+(buff[i+n]-buff[i])*k/n
                   int n,k; datetime ctime = iTime(NULL,TimeFrame,y);
                      for(n = 1; (i+n)<rates_total && time[i+n] >= ctime; n++) continue;
                      for(k = 1; k<n && (i+n)<rates_total && (i+k)<rates_total; k++)
                      {
                          _interpolate(cci);
  	                       if (cciUpa[i]!= EMPTY_VALUE) cciUpa[i+k] = cci[i+k];
  	                       if (cciUpb[i]!= EMPTY_VALUE) cciUpb[i+k] = cci[i+k];
  	                       if (cciDna[i]!= EMPTY_VALUE) cciDna[i+k] = cci[i+k];
  	                       if (cciDnb[i]!= EMPTY_VALUE) cciDnb[i+k] = cci[i+k];
                      }
         }
   return(rates_total);
   } 
   
   //
   //
   //
   //
   //
   
   double Kg = (3.0)/(2.0+RsxPeriod),Hg = 1.0-Kg;
   if (ArrayRange(work,0) != rates_total) ArrayResize(work,rates_total);
   for(r=rates_total-i-1; i>=0; i--,r++)
   {
         work[r][12] = iMA(NULL,0,1,0,MODE_SMA,RsxPrice,i);
         if (i==(rates_total-1)) { for (int c=0; c<12; c++) work[r][c] = 0; continue; }  

         //
         //
         //
         //
         //
      
         double mom = work[r][12]-work[r-1][12];
         double moa = fabs(mom);
         for (int k=0; k<3; k++)
         {
            int kk = k*2;
               work[r][kk+0] = Kg*mom           + Hg*work[r-1][kk+0];
               work[r][kk+1] = Kg*work[r][kk+0] + Hg*work[r-1][kk+1]; mom = 1.5*work[r][kk+0] - 0.5 * work[r][kk+1];
               work[r][kk+6] = Kg*moa           + Hg*work[r-1][kk+6];
               work[r][kk+7] = Kg*work[r][kk+6] + Hg*work[r-1][kk+7]; moa = 1.5*work[r][kk+6] - 0.5 * work[r][kk+7];
         }
         prices[i] = (moa != 0) ? iATR(NULL,0,AtrPeriod,i)*(fmax(fmin((mom/moa+1.0)*50.0,100.00),0.00)-50): 0; 
         double avg = 0; for(int k=0; k<CCIPeriod && (i+k)<rates_total; k++) avg +=      prices[i+k];      avg /= CCIPeriod;
         double dev = 0; for(int k=0; k<CCIPeriod && (i+k)<rates_total; k++) dev += fabs(prices[i+k]-avg); dev /= CCIPeriod;
         cci[i] = (dev!=0) ? (prices[i]-avg)/(0.015*dev): 0; 
      
         trend[i] = (i<rates_total-1) ? (cci[i]>OverBought) ? 1 : (cci[i]<OverSold) ? -1 : (cci[i]<OverBought && cci[i]>OverSold) ? 0 : trend[i+1] : 0; 
         slope[i] = (i<rates_total-1) ? (cci[i]>cci[i+1])   ? 1 : (cci[i]<cci[i+1]) ? -1 : slope[i+1] : 0;
         cciUpa[i] = (trend[i] == 1 && slope[i] == 1) ? cci[i] : EMPTY_VALUE;
         cciUpb[i] = (trend[i] == 1 && slope[i] ==-1) ? cci[i] : EMPTY_VALUE;
         cciDna[i] = (trend[i] ==-1 && slope[i] ==-1) ? cci[i] : EMPTY_VALUE;
         cciDnb[i] = (trend[i] ==-1 && slope[i] == 1) ? cci[i] : EMPTY_VALUE;       
   }
return(rates_total);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}
int timeFrameValue(int _tf)
{
   int add  = (_tf>=0) ? 0 : MathAbs(_tf);
   if (add != 0) _tf = _Period;
   int size = ArraySize(iTfTable); 
      int i =0; for (;i<size; i++) if (iTfTable[i]==_tf) break;
                                   if (i==size) return(_Period);
                                                return(iTfTable[(int)MathMin(i+add,size-1)]);
}
